% LOCAL LEARNING and MIXTURE MODELING OF ACTIVE BASIS
%% Load in exponential model, mex C codes, and set parameters
% ExponentialModel; % this line is not needed if 'storedExponentialModel' exists
clear; close all; 
load 'storedExponentialModel'; % load in exponential model 
mex CsharedSketch.c;  % learning from aligned images 
mex CgetSUM2.c; % code for finding nearest neighbors, without inhibition, so very fast
mex CgetMAX1.c; % local maximum pooling for MAX1 maps
mex Csigmoid.c; % sigmoid transformation 
mex Ccopy.c; % copy around detect location
epsilon = .1; % allowed correlation between selected Gabors 
subsample = 1; % subsample in computing MAX1 and SUM2 maps   
locationShiftLimit = 3; % shift in normal direction = locationShiftLimit*subsample pixels
orientShiftLimit = 1; % shift in orientation
Correlation = CorrFilter(allFilter, epsilon); % correlation between filters 
numNeighbor = 20; % number of nearest neighbors including self
numElement = 60; % number of Gabors in active basis   
numIteration = 3;  % number of iterations
factorHorizontal = 4; 
numVertical = 8; 
NEGMAX = -1e10; 
mkdir('working'); 
%% Load in training images
sizey = 140; % forced horizontal size of input image
imageFolder = 'positiveImage'; % folder of training images  
imageName = dir([imageFolder '\*.jpg']);
numImage = size(imageName, 1); % number of training images 
numHorizontal = floor(numImage/factorHorizontal); 
I = cell(1, numImage); 
maxSizex = 0; 
for (img = 1 : numImage)
    tmpIm = imread([imageFolder '\' imageName(img).name]); 
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    [sx, sy] = size(tmpIm); 
    sizex = round(sx*sizey/sy); 
    if (maxSizex<sizex)
        maxSizex = sizex; 
    end
    I{img} = imresize(single(tmpIm), [sizex sizey], 'nearest'); 
end
sizex = maxSizex; 
allSizex = zeros(1, numImage) + sizex; 
allSizey = zeros(1, numImage) + sizey; 
disp(['number of training images = ' num2str(numImage)]);
%% Compute SUM1 maps and MAX1 maps
disp(['start filtering']); tic
[SUM1map, Ifill] = ApplyFilterfft(I, allFilter, localOrNot, localHalfx, localHalfy, thresholdFactor); % SUM1 maps by Gabor filtering
disp(['filtering time: ' num2str(toc) ' seconds']);
Csigmoid(numImage, allSizex, allSizey, numOrient, saturation, SUM1map);
MAX1map = cell(numImage, numOrient);
for (img = 1:numImage)
    for (orient = 1:numOrient) 
        MAX1map{img, orient} = single(zeros(floor(sizex/subsample), floor(sizey/subsample)));
    end
end
CgetMAX1(numImage, allSizex, allSizey, numOrient, locationShiftLimit, orientShiftLimit, subsample, ...
         SUM1map, MAX1map);           
SUM1mapLearn = cell(numNeighbor, numOrient); 
for (img = 1:numNeighbor)
    for (orient = 1:numOrient)
        SUM1mapLearn{img, orient} = single(zeros(sizex, sizey)); 
    end
end
%% Prepare output variables for learning
selectedOrient = zeros(1, numElement);  % orientation and location of selected Gabors
selectedx = zeros(1, numElement); 
selectedy = zeros(1, numElement); 
selectedlambda = zeros(1, numElement); % weighting parameter for scoring template matching
selectedLogZ = zeros(1, numElement); % normalizing constant
commonTemplate = single(zeros(sizex, sizey)); % template of active basis 
deformedTemplate = cell(1, numNeighbor); % templates for training images 
for (img = 1 : numNeighbor)
    deformedTemplate{img} = single(zeros(sizex, sizey));  
end
SUM2score = zeros(numImage, 1); % template matching scores for training images 
%% start local learning from all the available examples
locationShiftLimit0 = 0; orientShiftLimit0 = 0; 
numImage0 = 1; 
SUM1map0 = cell(1, numOrient); 
for (orient = 1 : numOrient)
       SUM1map0{1, orient} = single(zeros(sizex, sizey));
end
available = zeros(numImage, 1)+1; 
for (vv  = 1 : numVertical)
showon = -1; bestSUM2score = NEGMAX; 
for (hh = 1 : numHorizontal)
  this = floor(rand(1)*numImage) + 1; 
  if (available(this)>0.)
    ABlearnThis; 
    meanSUM2 = mean(SUM2scoreSort(1:numNeighbor)); 
    if (meanSUM2>bestSUM2score)
        bestSUM2score = meanSUM2; 
        bestThis = this; 
    end
  end
end
%% select the best cluster, and remove the images from available examples 
this = bestThis; showon = 1; 
ABlearnThis; 
templateName = ['working/learnedTemplate' num2str(vv)]; 
save(templateName, 'selectedOrient', 'selectedx', 'selectedy', 'selectedlambda', 'selectedLogZ'); 
for (k = 1 : numNeighbor)
     neighbork = ind(k); 
     available(neighbork) = -1; 
end
disp(['--------------------------------------------------']); 
end
ABmixture; 

